from typing import Any, Dict, List, Tuple

import regex as re

from src.detections import EmailDetection
from src.detectors.interface import Detector


class EmailDetector(Detector):
    def __init__(self) -> None:
        super().__init__()

    def detect(
        self, text: str, annotations: Dict[str, List[Tuple[int, int, Any]]]
    ) -> List[Tuple[int, int, EmailDetection]]:
        return detect_emails(text)


EMAIL_REGEX = re.compile(
    r"(?P<local_part>[a-z0-9!#$%&\'*+/=?^_`{|}~-]+"
    r"(?:\.[a-z0-9!#$%&\'*+/=?^_`{|}~-]+)*@)"
    r"(?P<domain>(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+)"
    r"(?P<tld>[a-z0-9](?:[a-z0-9-]*[a-z0-9])?)",
    re.I,
)


def detect_emails(text: str) -> List[Tuple[int, int, EmailDetection]]:
    """
    Detects emails in the text.
    :param text: the text to be searched
    :type text: str
    :param language: the language of the text
    :type language: str
    :return: a list of tuples containing (start, end, entity_type)
    :rtype: List[Tuple[int, int, EmailAnnotation]]
    """
    matches = EMAIL_REGEX.finditer(text)
    emails = []
    for match in matches:
        emails.append((match.start(), match.end(), EmailDetection()))
    return emails
