from typing import Any, Dict, List, Tuple

import regex as re

from src.detections import PhoneNumberDetection
from src.detectors.interface import Detector


class PhoneNumberDetector(Detector):
    def __init__(self) -> None:
        super().__init__()

    def detect(
        self, text: str, annotations: Dict[str, List[Tuple[int, int, Any]]]
    ) -> List[Tuple[int, int, PhoneNumberDetection]]:
        return detect_phone_numbers(text)


PHONE_NUMBER_REGEX = re.compile(
    r"(?P<country_code>(00[1-9]\d?)|(\(?([+\d]{2,3})\)?)[- ]??)?"
    r"(?P<number>(\d[- ]??){9,10})"
)


def detect_phone_numbers(text: str) -> List[Tuple[int, int, PhoneNumberDetection]]:
    """
    Detects phone numbers in the text.
    :param text: the text to be searched
    :type text: str
    :param language: the language of the text
    :type language: str
    :return: a list of tuples containing (start, end, entity_type)
    :rtype: List[Tuple[int, int, PhoneNumberAnnotation]]
    """
    matches = PHONE_NUMBER_REGEX.finditer(text)
    phone_numbers = []
    for match in matches:
        phone_numbers.append((match.start(), match.end(), PhoneNumberDetection()))
    return phone_numbers
