from typing import List, Tuple

from src.detections import (CityDetection, CountryDetection, DateDetection,
                            Detection, EmailDetection, HydronymDetection,
                            KRSDetection, LocationDetection, NameDetection,
                            NumberDetection, OrganizationNameDetection,
                            PhoneNumberDetection, ProperNameDetection,
                            SerialNumberDetection, StreetNameDetection,
                            SurnameDetection, TINDetection, TitleDetection,
                            UrlDetection, UserDetection)
from src.replacers.interface import ReplacerInterface
from src.string_replacements import replace


class TagReplacer(ReplacerInterface):
    def __init__(self):
        self.tags_map = {
            NameDetection: "[OSOBA]",
            SurnameDetection: "[OSOBA]",
            StreetNameDetection: "[MIEJSCE]",
            LocationDetection: "[MIEJSCE]",
            CityDetection: "[MIEJSCE]",
            CountryDetection: "[MIEJSCE]",
            OrganizationNameDetection: "[ORGANIZACJA]",
            ProperNameDetection: "[NAZWA WŁASNA]",
            TitleDetection: "[TYTUŁ]",
            PhoneNumberDetection: "[CYFRY]",
            HydronymDetection: "[NAZWA WODNA]",
            UrlDetection: "[WWW]",
            UserDetection: "@[USER]",
            EmailDetection: "[MAIL]",
            DateDetection: "[DATA]",
            TINDetection: "[CYFRY]",
            KRSDetection: "[CYFRY]",
            SerialNumberDetection: "[NUMER IDENTYFIKACYJNY]",
            NumberDetection: "[CYFRY]",
        }

    def replace(
        self, text: str, detections: List[Tuple[int, int, Detection]]
    ) -> Tuple[str, List[Tuple[int, int, Detection]]]:

        result = [
            (start, end, self.tags_map.get(type(entity_type), "[OTHER]"))
            for start, end, entity_type in detections
        ]

        return replace(text, result), []
