from src.annotations import MorphosyntacticAnnotation, NerAnnotation
from src.detections import CityDetection, NameDetection, SurnameDetection
from src.detectors.ner import NerDetector


def test_ner_detector():
    detector = NerDetector(
        detection_mapping={
            "person_first_nam": "name",
            "person_last_nam": "surname",
            "city_nam": "city",
        }
    )

    annotations = [
        (10, 16, NerAnnotation("person_first_nam")),
        (100, 109, NerAnnotation("person_first_nam")),
        (30, 35, NerAnnotation("person_last_nam")),
        (50, 59, NerAnnotation("city_nam")),
        (120, 124, NerAnnotation("some_other_annotation")),
        (10, 16, MorphosyntacticAnnotation("1")),
        (100, 109, MorphosyntacticAnnotation("2")),
        (30, 35, MorphosyntacticAnnotation("3")),
        (120, 124, MorphosyntacticAnnotation("some_other_morphosyntactic_annotation")),
    ]

    result = detector.detect("", annotations)

    expected = [
        (10, 16, NameDetection(morpho_tag="1")),
        (100, 109, NameDetection(morpho_tag="2")),
        (30, 35, SurnameDetection(morpho_tag="3")),
        (50, 59, CityDetection(morpho_tag=None)),
    ]

    assert set(result) == set(expected)
