from tempfile import NamedTemporaryFile

from src.detections import CityDetection, NameDetection, SurnameDetection
from src.dictionaries.morphosyntactic.ner_file import \
    NERFileMorphosyntacticDictionary


def test_ner_file_morphosyntactic_dictionary():
    with NamedTemporaryFile() as file:
        file.writelines(
            [
                "name	Andrzejowi	Andrzej	a\n".encode("utf-8"),
                "name	Andrzej	Andrzej	b\n".encode("utf-8"),
                "name	Kasia	Kasia	c\n".encode("utf-8"),
                "city	Wrocław	Wrocław	d\n".encode("utf-8"),
                "city	Warszawa	Warszawa	c\n".encode("utf-8"),
                "city	Kraków	Kraków	d\n".encode("utf-8"),
            ]
        )
        file.flush()

        dictionary = NERFileMorphosyntacticDictionary(file.name)

        example_name_1 = NameDetection(morpho_tag="a")
        example_name_2 = NameDetection(morpho_tag="b")
        example_other = SurnameDetection(morpho_tag="c")

        assert dictionary.get_random_replacement(example_name_1) == "Andrzejowi"
        assert dictionary.get_random_replacement(example_name_2) == "Andrzej"

        # If no good replacement is present, it should choose a random one
        assert dictionary.get_random_replacement(example_other) in [
            "Andrzejowi",
            "Andrzej",
            "Kasia",
            "Wrocław",
            "Warszawa",
            "Kraków",
        ]
