from tempfile import NamedTemporaryFile

from src.detections import CityDetection, NameDetection, SurnameDetection
from src.dictionaries.morphosyntactic.ner_file_nkjp import \
    NERFileNKJPMorphosyntacticDictionary


def test_ner_file_nkjp_morphosyntactic_dictionary():
    with NamedTemporaryFile() as file:
        file.writelines(
            [
                "name	Andrzejowi	Andrzej	sg:dat:m1\n".encode("utf-8"),
                "name	Andrzej	Andrzej	sg:m1:imperf\n".encode("utf-8"),
                "name	Kasia	Kasia	sg:f:imperf\n".encode("utf-8"),
                "city	Wrocław	Wrocław	sg:m2:imperf\n".encode("utf-8"),
                "city	Warszawa	Warszawa	sg:f:imperf\n".encode("utf-8"),
                "city	Kraków	Kraków	sg:m2:imperf\n".encode("utf-8"),
            ]
        )
        file.flush()

        dictionary = NERFileNKJPMorphosyntacticDictionary(file.name)

        example_name_1 = NameDetection(morpho_tag="subst:sg:dat:m1")
        example_name_2 = NameDetection(morpho_tag="subst:sg:m1:imperf")
        example_other = SurnameDetection(morpho_tag="subst:sg:m1:imperf")

        assert dictionary.get_random_replacement(example_name_1) == "Andrzejowi"
        assert dictionary.get_random_replacement(example_name_2) in ["Andrzej", "Kasia"]

        # If no good replacement is present, it should choose a random one
        assert dictionary.get_random_replacement(example_other) in [
            "Andrzejowi",
            "Andrzej",
            "Kasia",
            "Wrocław",
            "Warszawa",
            "Kraków",
        ]
