from src.annotations import MorphosyntacticAnnotation, NerAnnotation
from src.input_parsers.wiktor_ner import WiktorNERInputParser

example_json = """{
    "filename": "test_filename",
    "text": "Marek Kowalski pojechał do Wrocławia.",
    "tokens": [
        {
            "index": 1,
            "position": [0,5],
            "orth": "Marek",
            "lexemes": [
                {
                    "lemma": "Marek",
                    "mstag": "subst:sg:nom:m1",
                    "disamb": true
                }
            ]
        },
        {
            "index": 2,
            "position": [6,14],
            "orth": "Kowalski",
            "lexemes": [
                {
                    "lemma": "Kowalski",
                    "mstag": "subst:sg:nom:m1",
                    "disamb": true
                }
            ]
        },
        {
            "index": 3,
            "position": [15,23],
            "orth": "pojechał",
            "lexemes": [
                {
                    "lemma": "pojechać",
                    "mstag": "praet:sg:m1:perf",
                    "disamb": true
                }
            ]
        },
        {
            "index": 4,
            "position": [24,26],
            "orth": "do",
            "lexemes": [
                {
                    "lemma": "do",
                    "mstag": "prep:gen",
                    "disamb": true
                }
            ]
        },
        {
            "index": 5,
            "position": [27,36],
            "orth": "Wrocławia",
            "lexemes": [
                {
                    "lemma": "Wrocław",
                    "mstag": "subst:sg:gen:m3",
                    "disamb": true
                }
            ]
        },
        {
            "index": 6,
            "position": [36,37],
            "orth": ".",
            "lexemes": [
                {
                    "lemma": ".",
                    "mstag": "interp",
                    "disamb": true
                }
            ]
        }
    ],
    "entities": [
        {
            "text": "Marek Kowalski",
            "type": "nam_liv",
            "tokens": [0, 2],
            "positions": [0, 14]
        },
        {
            "text": "Wrocławia",
            "type": "nam_loc",
            "tokens": [4, 5],
            "positions": [27, 36]
        }
    ]
}"""


def test_wiktor_ner_input_parser():
    parser = WiktorNERInputParser()

    text, annotations = parser.parse(example_json)

    assert text == "Marek Kowalski pojechał do Wrocławia."
    assert len(annotations) == 8

    assert (0, 14, NerAnnotation("nam_liv")) in annotations
    assert (27, 36, NerAnnotation("nam_loc")) in annotations

    assert (0, 5, MorphosyntacticAnnotation("subst:sg:nom:m1")) in annotations
    assert (6, 14, MorphosyntacticAnnotation("subst:sg:nom:m1")) in annotations
    assert (15, 23, MorphosyntacticAnnotation("praet:sg:m1:perf")) in annotations
    assert (24, 26, MorphosyntacticAnnotation("prep:gen")) in annotations
    assert (27, 36, MorphosyntacticAnnotation("subst:sg:gen:m3")) in annotations
    assert (36, 37, MorphosyntacticAnnotation("interp")) in annotations
