from src.detections import CityDetection, DateDetection, UserDetection
from src.replacers.user_replacer import UserReplacer


def test_user_replacer():
    text = "@zzzz32 urodziła sie 05.05.2005 we Wrocławiu"
    detections = [
        (0, 7, UserDetection()),
        (21, 31, DateDetection()),
        (35, 44, CityDetection()),
    ]

    replacer = UserReplacer()
    result = replacer.replace(text, detections)

    expected_text_ending = " urodziła sie 05.05.2005 we Wrocławiu"
    exptected_detections_left = [
        (len(result[0]) - 23, len(result[0]) - 13, DateDetection()),
        (len(result[0]) - 9, len(result[0]), CityDetection()),
    ]

    assert result[0].endswith(expected_text_ending)
    assert result[0][0 : -len(expected_text_ending)] != "@zzzz32"
    assert result[1] == exptected_detections_left


def test_user_replacer_same_user_same_replacement():
    text = "@zzzz32 @zzzz32 @aaaaa"
    detections = [
        (0, 7, UserDetection()),
        (8, 15, UserDetection()),
        (16, 22, UserDetection()),
    ]

    replacer = UserReplacer()
    result = replacer.replace(text, detections)

    old_users = text.split()
    new_users = result[0].split()

    assert old_users[0] != new_users[0]
    assert new_users[0] == new_users[1]
    assert result[1] == []
